/* 
 * Copyright (c) 2001-$Date: 2006-05-01 14:41:34 -0700 (Mon, 01 May 2006) $ TIBCO Software Inc. 
 * All rights reserved.
 * For more information, please contact:
 * TIBCO Software Inc., Palo Alto, California, USA
 * 
 * $Id: tibjmsMsgProducer.java 21731 2006-05-01 21:41:34Z rselph $
 * 
 */

/*
 * This is a simple sample of a basic tibjmsMsgProducer.
 *
 * This sample publishes specified message(s) on a specified
 * destination and quits.
 *
 * Notice that the specified destination should exist in your configuration
 * or your topics/queues configuration file should allow
 * creation of the specified topic or queue. Sample configuration supplied with
 * the TIBCO Enterprise Message Service distribution allows creation of any
 * destination.
 *
 * If this sample is used to publish messages into
 * tibjmsMsgConsumer sample, the tibjmsMsgConsumer
 * sample must be started first.
 *
 * If -topic is not specified this sample will use a topic named
 * "topic.sample".
 *
 * Usage:  java tibjmsMsgProducer  [options]
 *                               <message-text1>
 *                               ...
 *                               <message-textN>
 *
 *  where options are:
 *
 *   -server    <server-url>  Server URL.
 *                            If not specified this sample assumes a
 *                            serverUrl of "tcp://localhost:7222"
 *   -user      <user-name>   User name. Default is null.
 *   -password  <password>    User password. Default is null.
 *   -topic     <topic-name>  Topic name. Default value is "topic.sample"
 *   -queue     <queue-name>  Queue name. No default
 *
 */
/*
 * Slightly modified TIBCO example to automatically reconnect to the EMS server and act as a request-reply client
 * 2012-10-07 matti.kaikkonen@teliasonera.com
 * 
 */


import java.util.*;
import javax.jms.*;
import com.tibco.tibjms.Tibjms;

public class tibjmsMsgProducer implements Runnable, ExceptionListener 
{
	/*-----------------------------------------------------------------------
	 * Parameters
	 *----------------------------------------------------------------------*/

	String          serverUrl    = "tcp://localhost:7222,tcp://localhost:7222";
	String          userName     = null;
	String          password     = null;
	String          name         = "queue.sample";
	String    		replyName    = "sample";
	boolean         useTopic     = false;

	/*-----------------------------------------------------------------------
	 * Variables
	 *----------------------------------------------------------------------*/
	Connection      connection   = null;
	Session         session      = null;
	MessageConsumer msgConsumer = null;
	MessageProducer msgProducer = null;
	Message			replyMsg	= null;
	Destination     destination = null;
	Destination     replyDestination = null;
	boolean 		initialized = false;
	Thread 			t 			= null;
	Message 		msg			= null;
	boolean			closeRequested = false;


	public tibjmsMsgProducer(String[] args)
	{
		parseArgs(args);

		/*
        try {
            tibjmsUtilities.initSSLParams(serverUrl,args);
        }
        catch (JMSSecurityException e)
        {
            System.err.println("JMSSecurityException: "+e.getMessage()+", provider="+e.getErrorCode());
            e.printStackTrace();
            System.exit(0);
        }
		 */

		/* print parameters */
		System.err.println("\n------------------------------------------------------------------------");
		System.err.println("tibjmsMsgProducer SAMPLE");
		System.err.println("------------------------------------------------------------------------");
		System.err.println("Server....................... "+((serverUrl != null)?serverUrl:"localhost"));
		System.err.println("User......................... "+((userName != null)?userName:"(null)"));
		System.err.println("Destination.................. "+name);
		System.err.println("ReplyDestination............. "+replyName);
		//System.err.println("Message Text................. ");
		System.err.println("------------------------------------------------------------------------\n");

		/* initial connection */
		establishConnection();
		/* Create a separate thread to reestablish connection and reconnect */
		t = new Thread(this);
		t.start();

		/* Catching CTRL-C */
		Runtime.getRuntime().addShutdownHook(new Thread() {
			public void run() { 
				closeRequested = true;
				if (connection != null) {
					try {
						connection.close();
					} catch (JMSException e1) {
						e1.printStackTrace();
					}
				}
				System.err.println("\nShutting down! CTRL-C");

			}
		});

		/* request-reply loop - if there are connection issues then re-send request */
		do 
		{
			try  
			{
				/* Sleep while connection is closed */
				while (!initialized) Thread.sleep(100);

				/* Send message */
				((TextMessage) msg).setText("REQUEST Message");
				msgProducer.send(msg);
				System.err.println("Published message: "+msg);

				/* read reply */
				replyMsg = msgConsumer.receive();
				if (replyMsg != null) {
					System.err.println("Received message: "+ replyMsg);
				}

				/* For testing purposes sleep and loop forever */
				Thread.sleep(5000);

			} 
			catch (JMSException e) 
			{
				System.err.println("JMS EXCEPTION: "+ e.getMessage());
				// e.printStackTrace();
				// System.exit(-1);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}


		} while (!closeRequested);
	}


	/** 
	 * The separate reestablish connection thread's run method.
	 */
	public void run() {
		/* looping forever - same logic for initial connect and reconnect */
		do {
			try {
				if (!initialized && !closeRequested) 
					establishConnection();
				/* Sleep before checking if reconnect is needed */
				Thread.sleep(100);
			} catch (InterruptedException e) {}
		} while (true);
	}



	/*---------------------------------------------------------------------
	 * onException
	 *---------------------------------------------------------------------*/
	public void onException(JMSException e)
	{
		/* print the connection exception status */
		System.err.println("ECONNECTION EXCEPTION: "+ e.getMessage());

		if (connection != null) {
			try {
				connection.close();
			} catch (JMSException e1) {
				e1.printStackTrace();
			}
		}
		/* Reconnect needed */
		initialized = false;
	}


	/*---------------------------------------------------------------------
	 * establishConnection
	 *---------------------------------------------------------------------*/
	private void establishConnection() {
		/* Stay in this loop until connect succeeds */
		while(!initialized && !closeRequested) {
			try
			{
				ConnectionFactory factory = new com.tibco.tibjms.TibjmsConnectionFactory(serverUrl);
				/* set heartbeat to detect network issues faster */
				Tibjms.setPingInterval(10);

				/* create the connection */
				connection = factory.createConnection(userName,password);

				/* create the session */
				session = connection.createSession(false,javax.jms.Session.AUTO_ACKNOWLEDGE);

				/* set the exception listener */
				connection.setExceptionListener(this);

				/* create the destination */
				if(useTopic) {
					destination = session.createTopic(name);
					replyDestination = session.createTopic(replyName);
				}
				else {
					destination = session.createQueue(name);
					replyDestination = session.createQueue(replyName);
				}

				System.err.println("Producer to destination: "+name);
				System.err.println("Consumer to destination: "+replyName);

				/* create the producer */
				msgProducer = session.createProducer(destination);
				msg = session.createTextMessage();

				/* create the consumer */
				msgConsumer = session.createConsumer(replyDestination);

				/* start the connection */
				connection.start();

				/* Connect succeeded */
				initialized = true;
			}
			catch(Exception e)
			{
				System.err.println("CONNECTION EXCEPTION: "+ e.getMessage());
				// e.printStackTrace();
				try {
					/* Sleep before next connection attempt */
					Thread.sleep(1000);
				} catch (InterruptedException e1) {
					e1.printStackTrace();
				}
			}

		}

	}

	/*-----------------------------------------------------------------------
	 * usage
	 *----------------------------------------------------------------------*/
	private void usage()
	{
		System.err.println("\nUsage: java tibjmsMsgProducer [options] [ssl options]");
		//System.err.println("                                <message-text-1>");
		//System.err.println("                                [<message-text-2>] ...");
		System.err.println("\n");
		System.err.println("   where options are:");
		System.err.println("");
		System.err.println("   -server   <server URL>  - EMS server URL, default is local server");
		System.err.println("   -user     <user name>   - user name, default is null");
		System.err.println("   -password <password>    - password, default is null");
		System.err.println("   -queue    <queue-name>  - queue name, default is queue.sample");
		System.err.println("   -rqueue   <reply queue-name>  - reply queue name, default is sample");
		System.err.println("   -topic    <topic-name>  - topic name, no default");
		System.err.println("   -rtopic   <reply topic-name>  - reply topic name, no default");
		//System.err.println("   -help-ssl                 - help on ssl parameters");
		System.exit(0);
	}

	/*-----------------------------------------------------------------------
	 * parseArgs
	 *----------------------------------------------------------------------*/
	void parseArgs(String[] args)
	{
		int i=0;

		while(i < args.length)
		{
			if (args[i].compareTo("-server")==0)
			{
				if ((i+1) >= args.length) usage();
				serverUrl = args[i+1];
				i += 2;
			}
			else 
				if (args[i].compareTo("-topic")==0)
				{
					if ((i+1) >= args.length) usage();
					name = args[i+1];
					i += 2;
					useTopic = true;
				}
				else 
					if (args[i].compareTo("-rtopic")==0)
					{
						if ((i+1) >= args.length) usage();
						replyName = args[i+1];
						i += 2;
						useTopic = true;
					}
					else
						if (args[i].compareTo("-queue")==0)
						{
							if ((i+1) >= args.length) usage();
							name = args[i+1];
							i += 2;
						}
						else
							if (args[i].compareTo("-rqueue")==0)
							{
								if ((i+1) >= args.length) usage();
								replyName = args[i+1];
								i += 2;
							}
							else
								if (args[i].compareTo("-user")==0)
								{
									if ((i+1) >= args.length) usage();
									userName = args[i+1];
									i += 2;
								}
								else
									if (args[i].compareTo("-password")==0)
									{
										if ((i+1) >= args.length) usage();
										password = args[i+1];
										i += 2;
									}
									else
										if (args[i].compareTo("-help")==0)
										{
											usage();
										}
			/*							else
								if (args[i].compareTo("-help-ssl")==0)
								{
									tibjmsUtilities.sslUsage();
								}
								else
									if(args[i].startsWith("-ssl"))
									{
										i += 2;
									}
			 */									
		}
	}

	/*-----------------------------------------------------------------------
	 * main
	 *----------------------------------------------------------------------*/
	public static void main(String[] args)
	{
		tibjmsMsgProducer t = new tibjmsMsgProducer(args);
	}
}

